interface ActivityData {
  id: number
  user_id: number
  op_type: string
  act_user_id: number
  act_user: ActUser
  repo_id: number
  repo: Repo
  comment_id: number
  comment?: Comment
  ref_name: string
  is_private: boolean
  content: string
  created: string
}

interface ActUser {
  id: number
  login: string
  login_name: string
  source_id: number
  full_name: string
  email: string
  avatar_url: string
  html_url: string
  language: string
  is_admin: boolean
  last_login: string
  created: string
  restricted: boolean
  active: boolean
  prohibit_login: boolean
  location: string
  pronouns: string
  website: string
  description: string
  visibility: string
  followers_count: number
  following_count: number
  starred_repos_count: number
  username: string
}

interface Repo {
  id: number
  owner: Owner
  name: string
  full_name: string
  description: string
  empty: boolean
  private: boolean
  fork: boolean
  template: boolean
  parent: any
  mirror: boolean
  size: number
  language: string
  languages_url: string
  html_url: string
  url: string
  link: string
  ssh_url: string
  clone_url: string
  original_url: string
  website: string
  stars_count: number
  forks_count: number
  watchers_count: number
  open_issues_count: number
  open_pr_counter: number
  release_counter: number
  default_branch: string
  archived: boolean
  created_at: string
  updated_at: string
  archived_at: string
  permissions: Permissions
  has_issues: boolean
  external_tracker: ExternalTracker
  has_wiki: boolean
  wiki_branch: string
  globally_editable_wiki: boolean
  has_pull_requests: boolean
  has_projects: boolean
  has_releases: boolean
  has_packages: boolean
  has_actions: boolean
  ignore_whitespace_conflicts: boolean
  allow_merge_commits: boolean
  allow_rebase: boolean
  allow_rebase_explicit: boolean
  allow_squash_merge: boolean
  allow_fast_forward_only_merge: boolean
  allow_rebase_update: boolean
  default_delete_branch_after_merge: boolean
  default_merge_style: string
  default_allow_maintainer_edit: boolean
  avatar_url: string
  internal: boolean
  mirror_interval: string
  object_format_name: string
  mirror_updated: string
  repo_transfer: any
  topics: any
}

interface Owner {
  id: number
  login: string
  login_name: string
  source_id: number
  full_name: string
  email: string
  avatar_url: string
  html_url: string
  language: string
  is_admin: boolean
  last_login: string
  created: string
  restricted: boolean
  active: boolean
  prohibit_login: boolean
  location: string
  pronouns: string
  website: string
  description: string
  visibility: string
  followers_count: number
  following_count: number
  starred_repos_count: number
  username: string
}

interface Permissions {
  admin: boolean
  push: boolean
  pull: boolean
}

interface ExternalTracker {
  external_tracker_url: string
  external_tracker_format: string
  external_tracker_style: string
  external_tracker_regexp_pattern: string
}

interface Comment {
  id: number
  html_url: string
  pull_request_url: string
  issue_url: string
  user: any
  original_author: string
  original_author_id: number
  body: string
  assets: any[]
  created_at: string
  updated_at: string
}


class Codeberg {
  private cache: ActivityData[];
  private lastUpdate: Date | null = null;
  constructor() {
    this.cache = [];
  }

  private async refreshData(): Promise<Codeberg> {
    const response = await fetch("https://codeberg.org/api/v1/users/naomi-lgbt/activities/feeds?only-performed-by=true");
    const data = await response.json() as ActivityData[];
    this.cache = data;
    this.lastUpdate = new Date();
    return this;
  }

  public async getActivities(): Promise<ActivityData[]> {
    // Stale after 6 hours
    if (this.lastUpdate && (new Date().getTime() - this.lastUpdate.getTime()) < 6 * 60 * 60 * 1000) {
      return this.cache;
    }
    return this.refreshData().then(() => this.cache);
  }
}

const instantiated = new Codeberg();

export const getCodebergData = async (): Promise<ActivityData[]> => {
  return await instantiated.getActivities();
}
